#####################################
# Media Coverage of Trump: NYT, WSJ
# Text Classification: nb, lasso, svm 
# Michele Claibourn
# February 24, 2017
#####################################

#####################
# Loading libraries
# Setting directories
#####################
# install.packages("RTextTools")

rm(list=ls())
library(dplyr)
library(quanteda)
library(RTextTools)

# Load the data and environment from exploreNews.R
setwd("~/Box Sync/mpc/DataServices/dataForDemocracy/newspaper/factiva/")
# load("newspaperSentiment.RData") # original image
load("newspaperClassification.Rdata") # image containing end results 


###########
# Prep data
###########
# Trim words that appear in less than 0.5%/more than 99.5% of documents (to reduce the size of the matrix)
nydfmReduced <- dfm_trim(nydfm, min_docfreq = .005*ndoc(nydfm), max_docfreq = .995*ndoc(nydfm))
nydfmReduced

# Remove empty rows and confert to a tm document-term matrix
nydfmReduced <- nydfmReduced[which(rowSums(nydfmReduced) > 0),]
nydtm <- convert(nydfmReduced, to="tm")
nydtm

# Convert this to matrix form
nymat <- as.matrix(nydtm)

# Add response variable and some document variables to be used as additional predictors
nymat2 <- cbind(nymat, nymeta2$readability, nymeta2$pertone)

# Sample articles from the dtm to use as training set
set.seed(121)
trainDTM <- nymat2[sample(ndoc(nydfm), 1500), ] 
# Remaining articles used as testing set
set.seed(121)
testDTM <- nymat2[-sample(ndoc(nydfm), 1500), ] 

# Response vectors
set.seed(121)
trainY <- nymeta2$pub[sample(ndoc(nydfm), 1500)] # for training set
set.seed(121)
testY <- nymeta2$pub[-sample(ndoc(nydfm), 1500)] # for test set


#############
# Naive Bayes
#############
library(e1071)
# Use all available predictors to classify source
nbModel <- naiveBayes(trainDTM, trainY, laplace = 1)
nbPred <- predict(nbModel, newdata=testDTM) # use model to predict test set source
table(nbPred) # predicted source
table(nbPred, testY) # accuracy
sum(nbPred == testY)/length(testY) # proportion correct
table(testY)


#################################
# Logit with Lasso Regularization
#################################
library(glmnet)
# alpha=1 is lasso; alpha=0 is ridge regression
# perform cross-validation and find the best lambda (ie, lambda with lowest misclassification error);
# type.measure="class" gives misclassification error rate
set.seed(1017)
cv.lasso <- cv.glmnet(trainDTM, trainY, alpha=1, family="binomial", type.measure = "class")
plot(cv.lasso)
# what are the dotted vertical lines?
# lambda.min - value of log(lambda) that gives misclassification error rate.
# lambda.1se - largest value of log(lambda) such that error is within 1 standard error of the minimum
cv.lasso$lambda.min # best lambda as selected by cross validation
log(cv.lasso$lambda.min) # what's in the plot
# Examine effect of predictors
lassoCoef <- as.data.frame(as.matrix(coef(cv.lasso, s="lambda.min")))
lassoCoef$varname <- row.names(lassoCoef) # put row names into column
names(lassoCoef)[1] <- "weight"
lassoCoef <- lassoCoef %>% arrange(weight) # sort by weights
head(lassoCoef, 10) # more predictive of NYT
tail(lassoCoef, 10) # more predictive of WSJ

# Validate on test set
lassoPred <- predict(cv.lasso, newx = testDTM, s = "lambda.min", type="class") # use model to predict test set source
table(lassoPred, testY) # accuracy
sum(lassoPred == testY)/length(testY) # proportion correct

# Confusion matrix (one approach)
matrix(c(sum(lassoPred=="NYT" & testY=="NYT")/length(testY), 
         sum(lassoPred=="WSJ" & testY=="NYT")/length(testY), # false negatives
         sum(lassoPred=="NYT" & testY=="WSJ")/length(testY), # false positives
         sum(lassoPred=="WSJ" & testY=="WSJ")/length(testY)), 
       nrow=2, byrow=T,
       dimnames=list(c("pred=NYT","pred=WSJ"),
                     c("actual=NYT", "actual=WSJ")))


#########################
# Support Vector Machine
#########################
library(e1071)
trainDF <- as.data.frame(trainDTM)
names(trainDF)[8265:8266] <- c("readability", "pertone")
trainDF <- cbind(trainY, trainDF)
names(trainDF)[1] <- "Y"

# timing
t <- proc.time()
svmTest <- svm(Y ~ ., data=trainDF, kernel="linear", cost=100)
proc.time() - t # 2 minutes
# Find optimal parameters (for linear kernal, only cost, a penalization parameter)
t <- proc.time()
tuneSVM <- tune.svm(Y ~ ., data=trainDF, kernel="linear", cost=2^(2:10))
proc.time() - t # ~ 4.5 hours

tunedCost <- tuneSVM$best.model$cost # select optimal paramters

# Train SVM
t <- proc.time()
svmModel <- svm(Y ~ ., data=trainDF, kernel="linear", cost=tunedCost)
proc.time() - t # ~ 2 minutes

# Validate: performance on training set
sum(svmModel$fitted == trainDF$Y) / nrow(trainDF)  # training accuracy
table(source=trainDF$Y, predicted=svmModel$fitted)

# Validate: performance on test set 
testDF <- as.data.frame(testDTM)
names(testDF)[8265:8266] <- c("readability", "pertone")

predicted <- predict(svmModel, testDF)
# Confusion matrix (another approach)
confMat <- table(source=testY, predicted)
confMat

acc <- data.frame(t(as.vector(confMat)))
colnames(acc) <- c('TN','FN','FP','TP')
(accuracy <- (acc$TP + acc$TN) / nrow(testDF)) 
(precision <- acc$TP / (acc$TP + acc$FP))
(recall <- acc$TP / (acc$TP + acc$FN))  # sensitivity, Power
(Fscore <- 2 * (precision*recall) / (precision+recall))

# Save
save.image("newspaperClassification.Rdata")
