###################################
# Media Coverage of Trump: NYT, WSJ
# Initial Exploration 
# Michele Claibourn
# February 21, 2017
##################################

#####################
# Loading libraries
# Setting directories
#####################
# install.packages("scales")
# install.packages("wordcloud")
# install.packages("RColorBrewer")

rm(list=ls())
library(dplyr)
library(tidyr)
library(ggplot2)
library(quanteda)
library(scales)
library(wordcloud)
library(RColorBrewer)

# Load the data and environment from readNews.R
setwd("~/Box Sync/mpc/DataServices/dataForDemocracy/newspaper/factiva/")
# load("newspaper.Rdata") # original image
load("newspaperClassification.Rdata") # image containing end results


##################################
# Exploring the text, pre-analysis
##################################
summary(qcorpus, showmeta = TRUE)
nymeta <- docvars(qcorpus)

# Some descriptives
summary(as.integer(nymeta$wordcount))
qcorpus2 <- corpus_subset(qcorpus, as.integer(wordcount)>99)

table(nymeta$pub)
nymeta$pub <- as.factor(nymeta$pub)
nymeta$date <- as.Date(nymeta$datetimestamp)
nymeta2 <- subset(nymeta, as.integer(wordcount)>99)

# Number of stories by day
p <- ggplot(nymeta, aes(x=date)) 
p + stat_count(aes(fill=..count..), geom="point") + 
  facet_wrap(~ pub) +
  scale_x_date(labels = date_format("%m/%d"), date_breaks="1 day") + 
  theme(axis.text.x=element_text(angle=90), 
        plot.title = element_text(face="bold", size=20, hjust=0), 
        axis.title = element_text(face="bold", size=16),
        legend.position="none")

## Clean up terms
# common bigrams
bigrams <- collocations(qcorpus2, n = 500, size = 2)
# exclude collocations with stop words
bigrams <- removeFeatures(bigrams, stopwords("SMART"))
bigrams
# replace the top two phrases with single-token versions
qcorpus3 <- phrasetotoken(qcorpus2, bigrams[1:2])

## Frequent words/Wordclouds
nydfm <- dfm(qcorpus3, remove = c(stopwords("SMART")), 
             removePunct = TRUE, verbose=TRUE) # turn it into a document-feature matrix
topfeatures(nydfm, 20)
palD <- brewer.pal(8, "Dark2")
textplot_wordcloud(nydfm, max.words = 300, colors = palD, scale = c(4, .5))

# Wordcloud by source
bypaperdfm <- dfm(qcorpus3, groups = "publisher",
                  remove = c(stopwords("SMART")),
                  removePunct = TRUE, verbose = TRUE)

palO <- colorRampPalette(brewer.pal(9,"Oranges"))(32)[13:32]
textplot_wordcloud(bypaperdfm[1], max.words = 200, colors = palO, scale = c(4, .5)) # wsj

palB <- colorRampPalette(brewer.pal(9,"Blues"))(32)[13:32]
textplot_wordcloud(bypaperdfm[2], max.words = 200, colors = palB, scale = c(4, .5)) # nyt

## Key words in context (from quanteda)
kwic(qcorpus3, "tweet", 3)
test <- kwic(qcorpus3, "white", 3)
test

## Readability/Complexity Analysis (from quanteda)
fk <- textstat_readability(qcorpus3, measure = "Flesch.Kincaid")
nymeta2$readability <- fk

# Plot
p <- ggplot(nymeta2, aes(x = date, y = readability))
p + geom_jitter(aes(color=pub), alpha=0.5, width=0.25, height=0.0, size=2) +
  scale_x_date(labels = date_format("%m/%d"), date_breaks="1 day") +
  ggtitle("'Readability' of Newspaper Coverage of Trump") +
  labs(y = "Readability (grade level)", x = "Date of Article") +
  scale_color_manual(values=c("blue3","orange3"), name="Source") +
  theme(plot.title = element_text(face="bold", size=20, hjust=0),
        axis.title = element_text(face="bold", size=16),
        panel.grid.minor = element_blank(), legend.position = c(0.95,0.9),
        axis.text.x = element_text(angle=90),
        legend.text=element_text(size=10))

nymeta2 %>% group_by(pub) %>% summarize(mean(readability)) 


# Save
rm("p", "fk", "bigrams", "test", "palD", "palO", "palB")
save.image("newspaperExplore.RData")
