###################################
# Locating Quant Courses
# Text Classification (RTextTools) 
# Michele Claibourn
# February 26, 2017
##################################

#####################
# Loading libraries
# Setting directories
#####################
rm(list=ls())
setwd("~/Box Sync/mpc/DataServices/dataForDemocracy/classificationWorkshop/classificationMaterials/")
load("courseClassification.Rdata") # image created by this script

library(dplyr)
library(readr)
library(tm)
library(quanteda)
library(RTextTools)


#############
# Import data
#############
# Read in downloaded files
loufiles <- DirSource(pattern=".csv")
loulist <- do.call("rbind", lapply(loufiles$filelist, function(x) {
  dat <- read_csv(x)
  dat$file <- tools::file_path_sans_ext(basename(x))
  dat
}))

loulist$number <- as.factor(loulist$Number)                      # create course number as factor
loulist <- loulist %>% 
  mutate(quant = ifelse(Mnemonic=="STAT", 1, 
                        ifelse(grepl("^REL", Mnemonic), 0, NA))) %>%  # code stats classes as quant, religious studies as not
  mutate(sem=recode(file,                                             # create variable for semester
                    "CS1158Data"="Fall15", "CS1162Data"="Spring16", "CS1168Data"="Fall16", "CS1117Data"="Spring17")) 

###################
# Create the corpus
# and dfm
###################
louCorpus <- corpus(loulist$Description, 
                    docnames=paste0(loulist$Mnemonic, "-", loulist$number, "-", loulist$Section, "-", loulist$sem))
summary(louCorpus)
docvars(louCorpus) <- loulist[, c("Mnemonic", "number", "Title", "sem", "quant")]

# Generate a document-term/feature matrix
louDFM <- dfm(louCorpus, toLower = TRUE, removePunct = TRUE,
              stem = TRUE, remove = stopwords("english"), verbose = TRUE) 
head(louDFM)

#####################
# Trainng and Testing
#####################
# Work with just the labeled/coded courses
quantIndex <- !is.na(loulist$quant)
sampleCourses <- loulist[quantIndex,] 
restCourses <- loulist[!quantIndex,] # NOT WORKING

sampDFM <- louDFM[quantIndex,]
restDFM <- louDFM[!quantIndex,]

# Divide coded data into training and testing sets
set.seed(777)
sampDocs <- 1:nrow(sampDFM)
trainDocs <- sample(sampDocs, .7*length(sampDocs), replace = FALSE) ## training set
testDocs <- sampDocs[-sample(sampDocs, .7*length(sampDocs), replace = FALSE)]

# Create container/list to be used in classification algorithms
cont <- create_container(sampDFM, sampleCourses$quant,
                         trainSize = trainDocs, testSize = testDocs, virgin = FALSE)
str(cont)


#######################
# Train, test, evaluate
# RTextTools
#######################
# a. Support Vector Machines
svmModel <- train_model(cont, "SVM") # train the model
svmResult <- classify_model(cont, svmModel) # apply to test set
svmAnalytic <- create_analytics(cont, svmResult) # generate summaries
summary(svmAnalytic)
svmDoc <- svmAnalytic@document_summary

# b. Logistic with lasso regularization
glmModel <- train_model(cont, "GLMNET")
glmResult <- classify_model(cont, glmModel)
glmAnalytic <- create_analytics(cont, glmResult)
summary(glmAnalytic)
glmDoc <- glmAnalytic@document_summary

# c. Maximum entropy logistic
maxentModel <- train_model(cont, "MAXENT")
maxentResult <- classify_model(cont, maxentModel)
maxentAnalytic <- create_analytics(cont, maxentResult)
summary(maxentAnalytic)
maxentDoc <- maxentAnalytic@document_summary

# d. Classification tree
treeModel <- train_model(cont, "TREE")
treeResult <- classify_model(cont, treeModel)
treeAnalytic <- create_analytics(cont, treeResult)
summary(treeAnalytic)
treeDoc <- treeAnalytic@document_summary

# e. Random forest
rfModel <- train_model(cont, "RF")
rfResult <- classify_model(cont, rfModel)
rfAnalytic <- create_analytics(cont, rfResult)
summary(rfAnalytic)
rfDoc <- rfAnalytic@document_summary
save.image("courseClassification.Rdata")

# f. Boosted logit
boostModel <- train_model(cont, "BOOSTING")
boostResult <- classify_model(cont, boostModel)
boostAnalytic <- create_analytics(cont, boostResult) 
summary(boostAnalytic)
boostDoc <- boostAnalytic@document_summary
save.image("courseClassification.Rdata") 

# g. Linear Discriminant Analysis
# sldaModel <- train_model(cont, "SLDA")
# sldaResult <- classify_model(cont, sldaModel)
# sldaAnalytic <- create_analytics(cont, sldaResult)
# summary(sldaAnalytic)
# sldaDoc <- sldaAnalytic@document_summary

# h. Bootstrap aggregation (trees)
# baggingModel <- train_model(cont, "BAGGING")
# baggingResult <- classify_model(cont, baggingModel)
# baggingAnalytic <- create_analytics(cont, baggingResult)
# summary(baggingAnalytic)
# baggingDoc <- baggingAnalytic@document_summary

# i. NNET
# nnetModel <- train_model(cont, "NNET")
# nnetResult <- classify_model(cont, nnetModel) # didn't work
# nnetAnalytic <- create_analytics(cont, nnetResult)
# summary(nnetAnalytic)

#######################
# Additional assessment
#######################
# Confusion matrix examples
svmCM <- matrix(c(sum(svmDoc$MANUAL_CODE==1 & svmDoc$SVM_LABEL==1)/nrow(svmDoc), # true positives
                  sum(svmDoc$MANUAL_CODE==0 & svmDoc$SVM_LABEL==1)/nrow(svmDoc), # false positives
                  sum(svmDoc$MANUAL_CODE==1 & svmDoc$SVM_LABEL==0)/nrow(svmDoc), # false negatives
                  sum(svmDoc$MANUAL_CODE==0 & svmDoc$SVM_LABEL==0)/nrow(svmDoc)), # true negatives
                nrow=2, byrow=T,
                dimnames=list(c("pred=1","pred=0"),
                              c("actual=1", "actual=0")))
svmCM

boostCM <- matrix(c(sum(boostDoc$MANUAL_CODE==1 & boostDoc$LOGITBOOST_LABEL==1)/nrow(boostDoc), # true positives
                    sum(boostDoc$MANUAL_CODE==0 & boostDoc$LOGITBOOST_LABEL==1)/nrow(boostDoc), # false positives
                    sum(boostDoc$MANUAL_CODE==1 & boostDoc$LOGITBOOST_LABEL==0)/nrow(boostDoc), # false negatives
                    sum(boostDoc$MANUAL_CODE==0 & boostDoc$LOGITBOOST_LABEL==0)/nrow(boostDoc)), # true negatives
                  nrow=2, byrow=T,
                  dimnames=list(c("pred=1","pred=0"),
                                c("actual=1", "actual=0")))
boostCM
boostCM[1,1]/(boostCM[1,1]+boostCM[2,1]) # true positive rate/recall
boostCM[2,2]/(boostCM[2,2]+boostCM[1,2]) # true negative rate
boostCM[1,1] + boostCM[2,2] # accuracy

##########################
# Classify unlabeled cases
##########################
# Let's re-train the model on all of the labeled data
# (using our preferred models from above)
# and apply the result to the unlabeled data
# Create container/list to be used in classification algorithms
contLabeled <- create_container(sampDFM, sampleCourses$quant,
                                trainSize = 1:1640, virgin = FALSE)

# Estimate model
modelsLabeled <- train_models(contLabeled, algorithms=c("SVM", "GLMNET", "MAXENT", "TREE"))

# Apply model to unlabeled courses
predSize = nrow(restDFM) # a vector for labels
contUnlabeled <- create_container(restDFM, labels=rep(0,predSize), testSize=1:predSize, virgin=TRUE)
classifyUnlabeled <- classify_models(contUnlabeled, modelsLabeled)
ensembleUnlabeled <- create_scoreSummary(contUnlabeled, classifyUnlabeled)

# Attach predictions to initial data, look at results
coursesUnlabeled <- as.data.frame(restDFM)
coursesUnlabeled$id <- row.names(coursesUnlabeled)
coursesUnlabeled <- cbind(coursesUnlabeled, classifyUnlabeled)
table(coursesUnlabeled$GLMNET_LABEL)
subset(coursesUnlabeled, GLMNET_LABEL==1, select = c(id, GLMNET_PROB))

save.image("courseClassification.Rdata")

# Steps to improve performance:
# 1. Label a larger set (e.g., a complete semester)
# 2. Add document covariates (e.g., dept mnemonic, course level)

# Courses by semester
loulist$sem <- as.factor(loulist$sem)
louByCourseSem <- loulist %>%   # keep only one record per course per semester (i.e., get rid of "sections/labs" for primary course)
  group_by(Mnemonic, number, sem) %>% 
  summarize(sections=n(), students=sum(Enrollment), level=first(Number),
            title=as.character(first(Title)), 
            description=as.character(first(Description)))

table(louByCourseSem$sem) # courses numbers by semesters
table(loulist$sem) # course sections by semester